/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TCPCLIENT_H__
#define __TCPCLIENT_H__

#include "config.h"

#include <stdio.h>
#include <string>
#include <time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>
#include <map>
#include <vector>

#include "ringbuf.h"
#include "messagebus.h"
#include "timetracker.h"

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 64
#endif

// Arbitrary 16k ring.
#define CLI_RING_LEN (16384)

class TcpClient {
public:
    TcpClient();
    virtual ~TcpClient();

    // Register the message output stream
    virtual void RegisterMessageBus(MessageBus *in_msg) {
        msgbus = in_msg;
    }

    // Register the time events
    virtual void RegisterTimetracker(Timetracker *in_tracker) {
        timetracker = in_tracker;
    }

    // Is the client connection valid?
    virtual int Valid() {
        return cl_valid;
    }

    // Connect to a remote host
    virtual int Connect(const char *in_remotehost, short int in_port);

    // Kill the connection
    virtual int KillConnection();

    // Merge the existing fd into read and write sets as relevant
    virtual unsigned int MergeSet(fd_set in_rset, fd_set in_wset, 
                                  unsigned int in_max_fd,
                                  fd_set *out_rset, fd_set *out_wset);

    // Poll and dispatch to our parser
    virtual int Poll(fd_set& in_rset, fd_set& in_wset);

    // Return our info for callers who forgot what they told us to do
    virtual char *FetchRemoteHost();
    virtual short int FetchRemotePort();

    char *FetchError() {
        return errstr;
    }
    
protected:
    // Set the socket to be nonblocking io
    int SetNonblocking();

    // Low-level byte reading
    virtual int ReadBytes();

    // Low-level byte writing
    virtual int WriteBytes();

    // Parse the incoming data
    virtual int ParseData();

    MessageBus *msgbus;
    Timetracker *timetracker;
    
    char errstr[1024];

    // Initiate the connection in nonblocking mode - we always do, but 
    // subclasses like ssl can override
    int connect_nonblocking;
    
    int cl_valid;

    int client_fd;

    char remote_host[MAXHOSTNAMELEN];
    short int remote_port;

    RingBuffer *write_buf;
    RingBuffer *read_buf;

    struct sockaddr_in client_sock, local_sock;
    struct hostent *client_host;

};

#endif

