/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TCPMONITORCLIENT_H__
#define __TCPMONITORCLIENT_H__

#include "config.h"

#include <string>
#include <vector>

#include "tcpclient.h"
#include "macaddr.h"
#include "ap_monitor_proto.h"

class TcpMonitorClient : public TcpClient {
public:
    typedef struct zone {
        unsigned int id;
        unsigned int dirty;
        unsigned int num_sniffers;
        unsigned int num_rawtx;
        unsigned int num_ap;
        unsigned int num_nodes;
        unsigned int num_interfaces;
        unsigned int num_alerts;
        unsigned int num_attacks;
        time_t update_time;
    };

    typedef struct ap {
        char name[32];
        unsigned int zoneid;
        unsigned int stationid;
        unsigned int configured;
        unsigned int active;
        unsigned int dead;
        time_t last_helo;
        unsigned int num_interfaces;
        unsigned int loadavg_major;
        unsigned int loadavg_minor;
        time_t update_time;
    };

    typedef struct interface {
        char name[32];
        unsigned int stationid;
        unsigned int index;
        mac_addr mac;
        mac_addr innate_mac;
        unsigned int channel;
        unsigned int cf_bitmap;
        time_t last_mac_change;
        time_t num_clients;
        unsigned long long int framecount;
        unsigned long long int errorcount;
        unsigned long long int tx_packetcount;
        unsigned long long int rx_packetcount;
        unsigned long long int tx_bytecount;
        unsigned long long int rx_bytecount;
        time_t update_time;
    };

    TcpMonitorClient();
    virtual ~TcpMonitorClient();

    time_t FetchStartTime() {
        return start_time;
    }

    // Fetch info
    virtual void FetchTimeLimits(unsigned int *ret_conf, unsigned int *ret_reconf,
                                 unsigned int *ret_dead, unsigned int *ret_smite,
                                 unsigned int *ret_smiteignore);

    virtual void FetchInterfaceAllocation(unsigned int *ret_sniffers, 
                                          unsigned int *ret_rawtx,
                                          mac_addr *ret_basebssid,
                                          char *ret_basessid);

    virtual int FetchZoneDirty() {
        return zone_dirty;
    }

    virtual int FetchApDirty() {
        return ap_dirty;
    }

    virtual int FetchIfDirty() {
        return if_dirty;
    }

    virtual map<unsigned int, TcpMonitorClient::zone *> *FetchZoneMapPtr() {
        return &zone_id_map;
    }

    virtual map<unsigned int, TcpMonitorClient::ap *> *FetchApMapPtr() {
        return &ap_id_map;
    }

    virtual map<unsigned int, TcpMonitorClient::interface *> *FetchIfMapPtr() {
        return &if_id_map;
    }

protected:
    virtual int ParseData();

    // queue data
    // virtual int QueueData(const char *in_line);
  
    unsigned long int tx_seqid;
    unsigned long int rx_seqid;

    // Pushed via server
    time_t start_time;
    unsigned int config_delay;
    unsigned int reconfig_delay; 
    unsigned int deadidle_delay;
    unsigned int smite_delay;
    unsigned int smite_ignore;
    unsigned int sniffers_per;
    unsigned int rawtx_per;

    mac_addr base_bssid;
    char base_ssid[32];

    // Indexed by zone id
    map<unsigned int, TcpMonitorClient::zone *> zone_id_map;
    unsigned int zone_dirty;
    // Indexed by station id
    map<unsigned int, TcpMonitorClient::ap *> ap_id_map;
    unsigned int ap_dirty;
    // Games here - gen a unique key for an ap by shifting the 
    // station id and merging the index
    map<unsigned int, TcpMonitorClient::interface *> if_id_map;
    unsigned int if_dirty;
    
};

#endif

