/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TCPSSLCLIENT_H__
#define __TCPSSLCLIENT_H__

#include "config.h"

#include "tcpclient.h"

#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#include <openssl/ssl.h>
#include <openssl/x509v3.h>

#include "ssl_common.h"


// Password callbacks
extern char tcpcli_ctx_pem_passphrase[1024];
int tcpcli_pem_passwd_cb(char *buf, int size, int rwflag, void *password);

class TcpSSLClient : public TcpClient {
public:
    TcpSSLClient();
    virtual ~TcpSSLClient();

    // Set up the SSL connections too
    virtual int Connect(const char *in_remotehost, short int in_port);
    
    // Set SSL certs
    virtual int SetCTXCert(const char *in_certfile, const char *in_passphrase);
    // Set the verification paths
    virtual int SetCTXVerify(const char *in_cafile, const char *in_cadir);
    // Cipher setup
    virtual int SetCTXCipherList(const char *in_cipherlist);

    // Extra smarts like the server needed to handle FD set merging
    virtual unsigned int MergeSet(fd_set in_rset, fd_set in_wset,
                                  unsigned int in_max_fd,
                                  fd_set *out_rset, fd_set *out_wset);

    // Destroy the SSL structs
    virtual int KillConnection();

protected:
    // IO level
    virtual int ReadBytes();
    virtual int WriteBytes();

    // SSL IO and constructs
    SSL_CTX *con_ctx;
    SSL *con_ssl;

    // What are we in the middle of doing
    int ssl_busy_read;
    int ssl_busy_write;
    // What are we blocked waiting for
    int ssl_blocked_read;
    int ssl_blocked_write;
};

#endif

