/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TCPSSLSERVER_H__
#define __TCPSSLSERVER_H__

#include "config.h"

#include "tcpserver.h"

#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#include <openssl/ssl.h>
#include <openssl/x509v3.h>

#include "ssl_common.h"

// passwording for our cert file.  This isn't secure against pulling the pass out of
// memory, obviously.  Also, this is NOT THREAD SAFE
extern char tcpsrv_ctx_pem_passphrase[1024];
int tcpssl_pem_passwd_cb(char *buf, int size, int rwflag, void *password);

class TcpSSLServer : public TcpServer {
public:
    // SSL control objects structure
    typedef struct {
        // SSL record
        SSL *ssl_con;
        // Peer cert
        X509 *ssl_peercert;
        // Is the connection actually established?
        int ssl_established;

        // What are we in the middle of doing?
        int ssl_busy_read;
        int ssl_busy_write;
        // What are we blocked waiting for?
        int ssl_blocked_read;
        int ssl_blocked_write;
    } tcpssl_conrec;

    // Call the master setup stuff
    TcpSSLServer();
    virtual ~TcpSSLServer();

    // SSL certs
    virtual int SetCTXCert(const char *in_certfile, const char *in_passphrase);
    // Set the verification paths
    virtual int SetCTXVerify(const char *in_cafile, const char *in_cadir);
    // Cipher setup
    virtual int SetCTXCipherList(const char *in_cipherlist);

    // We need extra smarts to deal with SSLs oddities
    virtual unsigned int MergeSet(fd_set in_rset, fd_set in_wset, 
                                  unsigned int in_max_fd,
                                  fd_set *out_rset, fd_set *out_wset);

    // We need to clean up our SSL info
    virtual void KillConnection(int in_fd);

protected:
    // SSL error wrapper
    char *FetchSSLError();

    // SSL auth and mutual auth of certs
    virtual int Validate(int in_fd);

    // Handle nonblocking acceptance of the SSL
    int SSLNonBlockAccept(int in_fd);
    
    // SSL read/write
    virtual int ReadBytes(int in_fd);
    virtual int WriteBytes(int in_fd);

    // Map of FDs to SSL constructs
    map<int, TcpSSLServer::tcpssl_conrec *> ssl_rec_map;

    // SSL contexts
    SSL_CTX *serv_ctx;

};

#endif

